
package com.idca.fsim.core.jdbc;

import java.sql.Connection;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

/**
 * 
 * 
 * @author Willem Cazander
 * @version 1.0 Jan 4, 2006
 *
 */
public class JndiDataSourceProvider implements ConnectionProvider
{
	/** The context in which database datasources are found. */
	private static final String	DATASOURCE_CONTEXT	= "java:comp/env/jdbc/";	
	/** The connectionName for this Provider */
	private String connectionName = null;
	/** The jndi context to search for the dataSource. */
	private String dataSourceContext = null;
	/** The logger to log to. */
	private Logger logger = null;
	
	/**
	 * Creates an JndiDataSourceProvider and inits the Logger
	 */
	public JndiDataSourceProvider() {
		logger = Logger.getLogger(JndiDataSourceProvider.class.getName());
	}
	
	/**
	 * Creates JndiDataSourceProvider and sets its connectionName.
	 * @param connectionName
	 */
	public JndiDataSourceProvider(String connectionName) {
		super();
		setConnectionName(connectionName);
	}
	
	
	/**
	 * Gets an DataSource out of the jndi context.
	 * @param name	The name of the datasource in the jndi context.
	 * @return	The datasource.
	 */
	static public DataSource getDataSource(String name) {

		try {
			Context initialContext = new InitialContext();
			if(initialContext == null) {
				throw new RuntimeException("Init: Cannot get Initial Context");
			}
			DataSource datasource = (DataSource) initialContext.lookup(name);
			if(datasource == null) {
				throw new RuntimeException("Init: Cannot lookup datasource: '"+name+"'");
			}
			return datasource;
		} catch (NamingException ex) {
			throw new RuntimeException("Init: Cannot get connection " + ex);
		}
	}
	
	//========= ConnectionProvider
	
	/**
	 * Gets the connection name for which this 
	 * ConnectionProvider handles connections. 
	 * @return Returns the connection name.
	 * @see ConnectionProvider#getConnectionName()
	 */
	public String getConnectionName() {
		return connectionName;
	}
	
	/**
	 * Gets an connection of this ConnectionProvider.
	 * @return Returns an Connection
	 * @see ConnectionProvider#getConnection()
	 */
	public Connection getConnection() {
		try {
			if(dataSourceContext==null) {
				dataSourceContext=DATASOURCE_CONTEXT;
			}
			return getDataSource(dataSourceContext+connectionName).getConnection();
		} catch (Exception e) {
			logger.log(Level.WARNING,"Error while getting DataSource",e);
			throw new NullPointerException("No datasource");
		}
	}
	
	// ==== set/getter
	
	/**
	 * Sets the connectionName.
	 * @param connectionName The connectionName to set.
	 */
	public void setConnectionName(String connectionName) {
		this.connectionName=connectionName;
	}
	
	/**
	 * @return Returns the dataSourceContext.
	 */
	public String getDataSourceContext() {
		return dataSourceContext;
	}

	/**
	 * When set to null(default) then 
	 * DATASOURCE_CONTEXT is used. ("java:comp/env/jdbc/")
	 * @param dataSourceContext The dataSourceContext to set.
	 */
	public void setDataSourceContext(String dataSourceContext) {
		this.dataSourceContext = dataSourceContext;
	}
}